function t = binexpr2tree(expr)
% t = binexpr2tree(expr)
% Wandlung eines schaltalgebraischen Ausdrucks in einen Baum.
%
% expr :    String des vollstndig geklammerten schaltalgebraischen Ausdrucks 
%           in MATLAB-Syntax (berzhlige Klammern sind verboten)
%
% t    :    geschachtelter CellArray mit den Teilausdrcken der Form
%           {Operationssymbol, Operand1, Operand2, ... }
%           Folgende Operationssymbole in expr sind vorgesehen: 
%           '&' und '*' (AND), '|' (OR), '+' (XOR), '~' (NOT)
%           Im Baum werden Negationen von UND, OR, XOR zu
%           '/&', '/|', '/+'
%           
% Beispiel: binexpr2tree('(~a|b)&c&~(u|~v|w)') liefert 
%           {'&',{'|','~a','b'},'c',{'/|','u','~v','w'}}
%
% Siehe auch  tree2binexpr

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

if nargin<1  help binexpr2tree; break; end

% Gegebenenfalls ussere Klammer entfernen
% ========================================
ia = findstr('(',expr);
iz = findstr(')',expr);
if ~isempty(ia) & ~isempty(iz)
   if ia(1)==1 & iz(1)==length(expr)  expr = expr(2:end-1); end
end

% Konstanten
% ==========
opsym = '&*|+';                       % Operationssymbole

% Rekursionsende
% ==============
expr = reverse(deblank(reverse(deblank(expr))));
syv  = symvar(expr);
if strcmp(expr,syv)|strcmp(expr,['~' syv{1}])  t = expr; break; end

% Klammerhierarchie
% =================
kauf = cumsum(expr=='(');
kzu  = cumsum(expr==')');
kl   = kauf - [0 kzu(1:end-1)];       % Tiefe der Klammerhierarchie
ix0  = find(kl==0);                   % Indexbereich der Ebene 0
ix1  = find(kl>0);                    % Indexbereich der Ebenen>0
hie0 = expr(ix0);                     % Ausdruck auf der Ebene 0
k = 0;
while isempty(hie0)                   % falls berflssige Klammern
   k = k + 1;
   ix0  = find(kl==k);                % Indexbereich der Ebene k
   ix1  = find(kl>k);                 % Indexbereich der Ebenen>k
   hie0 = expr(ix0);                  % Ausdruck auf der Ebene k
end

% Ebene 0
% =======
exp0 = expr;
exp0(find(kl)) = '!';                 % Ausdruck der Ebene 0
le = length(exp0);
exp0 = strrep(exp0,'!!','!');
while length(exp0)<le
   le = length(exp0);
   exp0 = strrep(exp0,'!!','!');      % hhere Ebenen gekennzeichnet durch Symbol '!'
end
exp0 = [';' exp0 ';'];
for k=1:length(opsym)  
   nop(k) = sum(hie0==opsym(k));      % Anzahl der Operationssymbole
   exp0 = strrep(exp0,opsym(k),';');  % nur Variable brig mit Trenner ';' und '!'
end   
if sum(nop>0)>1  error('binexp2bintree:  Der Ausdruck ist nicht vollstndig geklammert!'); end
[nn,ix] = max(nop);

% Ausgabe
% =======
if ~isempty(ix1)                      % falls noch hhere Ebenen vorhanden
   dix = find(diff(ix1)>1);
   ia  = [ix1(1)  ix1(dix+1)];        % Anfnge der Segmente
   ie  = [ix1(dix)  ix1(end)];        % Ende der Segmente
end
if strcmp(exp0,';~!;')                % Sonderfall Negation in Ebene 0
   t = binexpr2tree(expr(ia(1)+1:ie(1)-1));
   t{1} = ['/' t{1}];
else
   t{1} = opsym(ix);                  % Operationssymbol auf Ebene 0
   if nn==0  t{1} = ' '; end          % keine Operation
   tx = findstr(';',exp0);            % Position der Trenner
   kk = 0;
   for k=1:length(tx)-1
      segm = exp0(tx(k):tx(k+1));
      segm = segm(2:end-1);
      if findstr('!',segm)
         kk = kk + 1;
         t{k+1} = binexpr2tree(expr(ia(kk)+1:ie(kk)-1)); % rekursiv behandeln
         if findstr('~',segm)  t{k+1}{1} = ['/' t{k+1}{1}]; end
      else
         t{k+1} = segm;               % Variable direkt (evtl. negiert)
      end
   end
end
