function fsmtab(stab,sname,z0,typ,xname,xcod,yname)
% fsmtab(stab,sname,z0,typ,xname,xcod,yname)
% Graphik der Flusstafel fr die bergangstabelle stab eines Schaltwerks.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index 0, so bleibt der Folgezustand leer, 
%          Ist der Index -1, so tritt anstelle des Folgezustands ein * .
%          stab kann auch ein StructureArray sein von zwei Tabellen:
%          die erste wie oben beschrieben fr die bergangsfunktion,
%          die zweite fr die Ausgabefunktion. Letztere ist im Falle 
%          eines Moore-Schaltwerks ein m-Spaltenvektor und im Falle 
%          eines Mealy-Schaltwerks eine m*n-Matrix von Indices in yname.
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben.
% z0    :  Index des Anfangszustands (Default 1)
% typ=0 :  getaktetes Scahltwerk
% typ=1 :  nicht-getaktetes Schaltwerk, stabile Zustnde markiert.
%          stab-Werte <1 fhren evtl. zu Fehlermeldungen.
% xname :  StructureArray mit den Strings der Inputnamen.
%          Hier kann man z.B. die Bitmuster angeben.
% xcod  :  String einer kurzen Erluterung zu xname, z.B. im 
%          Falle von Bitmustern 'D,C' um die Reihenfolge festzulegen.
% yname :  StructureArray mit den Strings der Ausgabecodierung oder -bezeichnung.
%
% Beispiel:  fsmtab([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'},3,1,{'0';'1'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = [1 2; 3 4; 2 3; 1 4]; end
if nargin<2  
   sname = {'A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';...
            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'};
   k = 1;   
   while length(sname)<length(stab)
      sname = cat(1,sname,strcat(sname{k},sname));
      k = k + 1;
   end
end
if nargin<3   z0 = 1;  end
if nargin<4  typ = 0;  end
if nargin<6 xcod = ''; end
if nargin<7 
   yname = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';...
            'n';'o';'p';'q';'r';'s';'t';'u';'v';'w';'x';'y';'z'};
end

% Konstanten
% ==========
if iscell(stab)
   yt   = 1;
   ytab = stab{2};           % Ausgangstabelle
   stab = stab{1};           % bergangstabelle
   [dummy,ny] = size(ytab)
else
   yt = 0;
   ny = 0;
end
[ns,nx] = size(stab);        % Anzahl von Knoten und mglichen Inputs
fsiz = min(15,500/ns);       % FontSize der Tabelle
fsz  = min(15,fsiz);         % FontSize der Beschriftung
ls = 0;
for k=1:length(sname) ls = max([ls,length(sname{k})]); end
fszc = min(fsiz-2,sqrt(2/ls)*fsz); % FontSize der Zustandsbezeichnung
lw = 1.5;                    % LineWidth
lc = 'k';                    % Linien-Farbe
nc = 0.7*[1 1 1];            % Knoten-Farbe
sc = 'g';                    % Farbe selektierter Knoten
nl = 'k';                    % Kanten-Farbe
sl = 'r';                    % Farbe selektierter Kanten
r  = 1/ns;
[msn,nsn] = size(sname);
if msn==1
   snm = cat(2,{'*',' '},sname);
else
   snm = cat(1,{'*';' '},sname);
end

% Hilfetext
% =========
helptext = {' ';
            ' ';
            'Hilfe zur Flusstafel';
            '=============';
            '';
            'nderung des Eingabeindex:';
            '   Mausklick auf den Eingabeindex';
            '   oder zugleich mit Takt:';
            '';
            'Fortschalten in Folgezustand:';
            '   Mausklick auf einen Folgezustand';
            '   des momentanen Zustandes'};

% Fenster
% =======
fh = fullscreen('Flusstafel','w',0,[max(0,0.81-0.03*nx) 1 0 1]);
axes('Position',[0 0 1 1])

% Achsenskalierung
% ================
axis([-2.5 nx+ny+2.5 -ns-1 +1]); axis off

% Flusstafel
% ==========
for k=1:ns
   for j=1:nx
      th(k,j) = text(j,-k,snm{2+stab(k,j)},'FontSize',fszc,'FontName','Helvetica',...
                     'ButtonDownFcn','fsmtabbdf',...
                     'HorizontalAlignment','center','VerticalAlignment','middle');
      if typ & stab(k,j)==k  drawcircle(j,-k,0.06*ns^0.7,'LineWidth',2,'Color','b'); end
   end
   if yt
      for j=1:ny
         text(nx+j+1,-k,yname{ytab(k,j)},'FontSize',fszc,'FontName','Helvetica',...
              'HorizontalAlignment','center','VerticalAlignment','middle');
      end
   end
end

% Beschriftung der Flusstafel
% ===========================
line([-1.8 nx+ny+1.5],[0 0]-0.5,'LineWidth',lw,'Color',lc)
line([0 0],[-ns-0.4 0.6],'LineWidth',lw,'Color',lc)
text(-1,0,'Zustand','FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
text(0.5+nx/2,0.5,'Index der Eingabe','FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
%if yt
%   if ny==1  ystr = '/y'; else ystr = '/Ausgabe'; end
%   text(1.5+nx+ny/2,0.5,ystr,'FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
%end
if nargin>4
   text(0.5+nx/2,0.25,['(darunter Codierung ' xcod ')'],'FontSize',fszc,'HorizontalAlignment','center')
end
for k=1:ns
   ths(k) = text(-1,-k,snm{k+2},'FontSize',fsz,'FontName','Helvetica',...
                 'HorizontalAlignment','center','VerticalAlignment','middle');
end
for j=1:nx
   thx(j) = text(j,0,sprintf('%d',j),'FontSize',fsz,'FontName','Helvetica',...
                 'ButtonDownFcn','fsmtabbdf',...
                 'HorizontalAlignment','center','VerticalAlignment','middle');
   if nargin>4  text(j,-0.25,xname{j},'FontSize',fszc,'FontName','Helvetica',...
                     'ButtonDownFcn','fsmtabbdf',...
                     'HorizontalAlignment','center','VerticalAlignment','middle');
                  
   end
end

% Interaktion
% ===========
fud.helptext = helptext;
fud.stab     = stab;
fud.currs    = z0;
fud.currx    = 1;
fud.ths      = ths;
fud.thx      = thx;
fud.th       = th;
fud.fig      = [];
fud.figfcn   = [];
set(fud.thx(fud.currx),'Color','r','FontWeight','bold')
set(fud.ths(fud.currs),'Color','g','FontWeight','bold')
set(fud.th(fud.currs,fud.currx),'Color','r','FontWeight','bold')
set(fh,'UserData',fud,'KeyPressFcn','helpfcn');
