function kvdiagram(sfcn,variable,loop,currloop)
% kvdiagram(sfcn,variable,loop, currloop)
% Interaktive Graphik des KV-Diagramms fr die durch sfcn 
% definierte Schaltfunktion mit den Variablen variable. 
%
% sfcn kann auf folgende Weise gegeben werden:
% sfcn = natrliche Zahl. In diesem Fall wird ein KV-Diagramm 
%        gezeichnet mit den Indices der Felder anstelle der 
%        Funktionswerte
% sfcn = Vektor der Lnge 2^n , der die Funktionstabelle enthlt
%        (0 bzw. 1 fr festgelegte Werte; <0 fr Don't Care
% sfcn = String der Lnge 2^n mit den Bits der Funktionstabelle 
%        ('0' bzw. '1' bzw. '*')
% sfcn = String eines Ausdrucks in den Variablen variable
%
% variable :  Cell array mit den Variablen als Strings
%             Die Variablen drfen nicht binexp oder variable
%             heissen. Auch die Schlsselwrter von MATLAB 
%             wie for, while, end, case etc. sind nicht erlaubt.
%             Dieser Parameter ist nur erforderlich im Fall,
%             dass sfcn als Ausdruck in diesen Variablen vorliegt.
%             Default ist  {'a'; 'b'; 'c'; 'd'}
% 
% Zur Darstellung schon gewhlter Schleifen dienen:
% 
% loop     :  CellArray mit den Listen der Schleifenteilnehmer
% currloop :  fokussierte Schleife
% Diese Daten knnen nach dem Whlen der Schleifen mit 
% get(gcf,'UserData') erfragt werden.
%
% Beispiele:
% kvdiagram(5)
% kvdiagram(randb(16),{'x_1';'x_2';'x_3';'x_4'})
% kvdiagram('(u&~v)|~w',{'u';'v';'w'})
%
% Interaktion
% ===========
% Disjunktive und konjunktive Form:
% Nach dem Aufruf von kvdiagram kann mit der Taste K oder k in die 
% konjugierte Form gewechselt werden.  Ein Wechsel zurck ist mit 
% der Taste D oder d mglich, wenn alle evtl. schon geklickten 
% Schleifen weggeklickt sind.
%
% Schleifen:
% Felder werden durch Anklicken hinzugefgt bzw. entfernt. 
% Eine neue Schleife beginnt man mit der Taste + .  Mit den 
% Tasten + und - kann man von Schleife zu Schleife wechseln 
% und dann in der angewhlten Schleife Korrekturen machen.
%
% Terme:
% Mit Klick auf einen der Schleifenbuttons in der rechten 
% Schleifenspalte wird der zugehrige Term berechnet.
%
% Minimierter Ausdruck:
% Mit der Taste r wird die erreichte dijunktive oder konjunktive 
% Form in MATLAB-Syntax im Kommandofenster ausgegeben.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  sfcn = 4; end  
if nargin<2
   if length(sfcn)==1  kk = sfcn; else kk = round(log2(length(sfcn))); end
   v  = 'abcdefghijklmnopqrstuvwxyz';
   for k=1:kk  variable{k} = v(k); end 
end

% Konstanten
% ==========
fws = 20;                                  % FixedWidthFontSize
fsz = 17;                                  % FontSize
col = color(1:64,0.7,3);
col = col(1+rem(1:4:2000,59),:);           % Feld-Farben
ss  = get(0,'ScreenSize');                 % Bildschirmauflsung

% Hifetext
% ========
helptext = {'Hilfe zu kvdiagram';
            '------------------------';
            '';
            'Editieren der Funktionswerte:';
            '   Nach Bedarf mit den Tasten 0 1 * einen Wert whlen';
            '   und nun den Wert mit der Maus in die Felder klicken.';
            '';
            'Whlen der konjunktiven bzw. disjunktiven Form:';
            '   Taste k bzw d tippen. Ein Wechsel ist nur mglich,';
            '   wenn keine Schleifen existieren.';
            '';
            'Bilden einer Schleife:';
            '   Mit der Maus gewhlte Felder anklicken.';
            '   Wiederholtes Klicken entfernt das Feld.';
            '';
            'Neue Schleife beginnen:';
            '   Mit der Taste + wird die nchste Schleife gewhlt,';
            '   mit der Taste - die vorige.';
            '';
            'Primimplikant anzeigen:';
            '   Klicken auf den farbigen Schleifenbutton (Spalte rechts).';
            '';
            'Minimierten Ausdruck in Matlab-Syntax und Aufwand ausgeben:';
            '   Taste r tippen.';
            '';
            'Entfernen des Bildes mit der Taste ENTF';
            '';
            'Entfernen aller Bilder mit der Taste ESC'};

% Berechnung der Funktionstabelle in f
% ====================================
if ischar(sfcn)
   if nargin==1                            % Definition als Ausdruck ?
      n = round(log(length(sfcn))/log(2)); % Anzahl der Variablen
      f = 3*ones(1,2^n);                   % Vorbesetzung mit Fehlercode
      f(findstr('0',sfcn)) = 0;            % Nullen in f setzen
      f(findstr('1',sfcn)) = 1;            % Einsen in f setzen
      f(findstr('*',sfcn)) =-1;            % Don'Care in f setzen
   else                                    % Definition als Funktionstabelle in String
      f = binexp2bintab(sfcn,variable);    % Berechnung der Funktionstabelle   
   end
elseif length(sfcn)==1
   f = 1:2^sfcn;                           % Indices statt Funktionstabelle
else
   f = sfcn;                               % Funktionstabelle ist direkt gegeben
end

% KV-Diagramm zeichnen
% ====================
fh = fullscreen('KV-Diagramm','w',0,[0 1 0.3 1]);
ax1= axes('Position',[0 0 ss(4)/ss(3) 1]);
axis square
p  = log2(length(f));
ph = drawkv([0 0 1 1/(1+rem(p,2))],f,variable,1:2^p);  % rekursiv zeichnen
if length(f)>8  s = 0.97; else s = 1.5; end
axis(s*[-1 +1 -1 +1]); axis off

% Minterm-Fenster
% ===============
fh2 = fullscreen('Minterme','w',0,[0 1 0 0.3]);
ax3 = axes('Position',[0 0 1 1]);
ph2 = patch([0 100 100 0],[-0.2 -0.2 2 2],col(1,:));
xa  = 10;
text(xa+1,1.5,'Feld-Nr.','FontSize',fsz)
text(1,0.5,'Variable','FontSize',fsz)
line([0 xa; 100 xa],[-0.2 -length(variable)-1; -0.2 2],'Color','k')
for k=1:length(variable)
   text(1,-k,variable(k),'FontSize',fsz)
   mth(k) = text(xa+1,-k,'','FontSize',fws,'FontName','FixedWidth');
end
mth(length(variable)+1) = text(xa+1,0.5,'','FontSize',fws,'FontName','FixedWidth');
axis([0 100 -length(variable)-0.5 2]); axis off

% Interaktion ermglichen
% =======================
figure(fh);
fud.variable = variable;
fud.tab = f;
fud.currloop = 1;
if nargin>2
   fud.loop = loop;
else
   fud.loop = {[]};
end
fud.ph = ph;
ax2 = axes('Position',[ss(4)/ss(3) 0 1-ss(4)/ss(3) 1]);
fud.ax = [ax1 ax2];
text(0,0,'Schleife  /  Term','FontSize',15)
ns = min(32,2^p);
dx = min(80/ns,10);
for k=1:ns
   lph(k) = patch(dx/2+[0 dx dx 0],-k-[0 0 1 1],col(k,:));  % Term-Feld
   lth(k) = text(2*dx,-k-0.5,'','FontSize',fsz);            % Term-Text
   set(lph(k),'UserData',k,'ButtonDownFcn','kvdiagrambdf')
end
set(lph(length(fud.loop)+1:end),'Visible','off')
axis([0 40 -ns-3 1]); axis off
fud.helptext = helptext;
fud.lph = lph;
fud.lth = lth;
fud.mth = mth;
fud.dmf = 1;
fud.ph2 = ph2;
set(fh,'UserData',fud,'KeyPressFcn','kvdiagramkpf')

if nargin>2
   fud.loop = loop;
   for k=1:length(loop)
      fud = get(fh,'UserData');
      fud.currloop = k;
      set(fh,'UserData',fud,'KeyPressFcn','kvdiagramkpf')
      kvdiagrambdf
   end
   if nargin>3  
      fud = get(fh,'UserData');
      fud.currloop = currloop;
      set(fh,'UserData',fud,'KeyPressFcn','kvdiagramkpf')
      kvdiagrambdf
   end
end

%--------------------------------------------------------------------------

function ph = drawkv(pos,fkv,variable,in)
% drawkv ist eine rekursive Hilfsfunktion zum Zeichnen eines KV-Diagramms
fsiz = 3 + 80/length(variable);
a = 0.7;
n = length(fkv);
p = round(log2(n));
m = length(variable);
k = m - p;
if n==1
   if fkv<0  s = '*'; else s = num2str(fkv); end
   [ph,th] = drawcsq(pos(1:2),2*pos(3),'w',s,fsiz-3);
   ptud.index = in;
   ptud.ph = ph;
   ptud.th = th;
   ptud.ls = [];
   set([ph th],'ButtonDownFcn','kvdiagrambdf','UserData',ptud)
else
   if rem(p,2)
      sx = pos(3)/2;
      sy = pos(4);
      if p==3  
         ix = [(5*n/8+1:6*n/8),(4*n/8+1:5*n/8),(7*n/8+1:n),(6*n/8+1:7*n/8)]; 
      else 
         ix = (n/2+1:n); 
      end
      ph = [drawkv([pos(1)-sx pos(2) sx sy],fkv(1:n/2),variable,in(1:n/2)), ...
            drawkv([pos(1)+sx pos(2) sx sy],fkv(ix),variable,in(ix))];
      ph = ph([1:n/2 ix]);
   else
      sx = pos(3);
      sy = pos(4)/2;
      if p>3  sx = a*sx; sy = a*sy; end
      if p==4  
         ix = [(5*n/8+1:6*n/8),(4*n/8+1:5*n/8),(7*n/8+1:n),(6*n/8+1:7*n/8)]; 
      else 
         ix = (n/2+1:n); 
      end
      ph = [drawkv([pos(1) pos(2)+sy sx sy],fkv(1:n/2),variable,in(1:n/2)), ...
            drawkv([pos(1) pos(2)-sy sx sy],fkv(ix),variable,in(ix))];
      ph = ph([1:n/2 ix]);
   end
   if (p==m)|(rem(p,4)==0)|((rem(p,2)==0)&p>4)
      u = 1.1;
      v = 1.12;
      if rem(p,2)
         line(pos(1)+[0 2*sx],pos(2)+u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1)+sx,pos(2)+v*sy,variable(k+1),'HorizontalAlignment','center', ...
              'VerticalAlignment','bottom','FontSize',fsiz)
      else
         line(pos(1)-u*sx+[0 0],pos(2)-[2*sy 0],'Color','k','LineWidth',1.5)
         text(pos(1)-v*sx,pos(2)-sy,variable(k+1),'HorizontalAlignment','right', ...
              'VerticalAlignment','middle','FontSize',fsiz)
         line(pos(1)+[0 sx],pos(2)+2*u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1)+sx/2,pos(2)+2*v*sy,variable(k+2),'HorizontalAlignment','center', ...
              'VerticalAlignment','bottom','FontSize',fsiz) 
      end
      if p==4
         line(pos(1)+0.5*sx*[-1 +1],pos(2)-2*u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1),pos(2)-2*v*sy,variable(k+4),'HorizontalAlignment','center', ...
              'VerticalAlignment','top','FontSize',fsiz)
         line(pos(1)+u*sx+[0 0],pos(2)-sy*[-1 +1],'Color','k','LineWidth',1.5)
         text(pos(1)+v*sx,pos(2),variable(k+3),'HorizontalAlignment','left', ...
              'VerticalAlignment','middle','FontSize',fsiz)         
      end
      if p==3
         line(pos(1)+sx*[-1 +1],pos(2)-u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1),pos(2)-v*sy,variable(k+3),'HorizontalAlignment','center', ...
              'VerticalAlignment','top','FontSize',fsiz)
         line(pos(1)-2*u*sx+[0 0],pos(2)-[sy 0],'Color','k','LineWidth',1.5)
         text(pos(1)-2*v*sx,pos(2)-sy/2,variable(k+2),'HorizontalAlignment','right', ...
              'VerticalAlignment','middle','FontSize',fsiz)
      end
   end
end
